-- core/init.lua :: The main file of the core system, an object full of subsystem
-- loaded by the game to handle the main functions (like screen, translation,
-- inputs…)

--[[
  Copyright © 2019 Kazhnuz

  Permission is hereby granted, free of charge, to any person obtaining a copy of
  this software and associated documentation files (the "Software"), to deal in
  the Software without restriction, including without limitation the rights to
  use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
  the Software, and to permit persons to whom the Software is furnished to do so,
  subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
  FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local CoreSystem = Object:extend()

local DebugSystem = require "core.debug"

local Options     = require "core.options"
local Input       = require "core.input"
local Screen      = require "core.screen"
local Lang        = require "core.lang"
local SceneManager= require "core.scenemanager"

function CoreSystem:new()
  self.debug    = DebugSystem(self)
  self.options  = Options(self)
  self.input    = Input(self)
  self.screen   = Screen(self)
  self.scenemanager = SceneManager(self)
end

function CoreSystem:mousemoved(x, y, dx, dy)
  local x, y    = self.screen:project(x, y)
  local dx, dy  = self.screen:project(dx, dy)
  self.scenemanager:mousemoved(x, y, dx, dy)
end

function CoreSystem:mousepressed( x, y, button, istouch )
  local x, y    = self.screen:project(x, y)
  self.scenemanager:mousepressed( x, y, button, istouch )
end

function CoreSystem:update(dt)
  self.debug:update(dt)
  self.input:update(dt)

  self.scenemanager:update(dt)
end

function CoreSystem:draw()
  self.scenemanager:draw()
end

function CoreSystem:exit()
  self.options:save()
  love.event.quit()
end

return CoreSystem
